#!/bin/zsh

# Copyright 2014 - 2016 Paul Hänsch
#
# This file is part of shcgi.
# 
# shcgi is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# shcgi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with shcgi.  If not, see <http://www.gnu.org/licenses/>. 

export LC_ALL=C LANG=C

# this program is supposed to be symlinked into a http root directory
# we will use the http root as object storage (data directory) and call sub
# programs from the directory in which the real executable resides
# therefore we need to identify the code and data directories _EXEC and _DATA
call="$0"
real="$(readlink -f $call)"
export _DATA="$(dirname "$call")"  #storage directory
export _EXEC="${real%/shcgi/index.cgi}"  #execution directory

# put debug options in the local.opts file
. "$_EXEC/shcgi/misc.sh"
. "$_EXEC/shcgi/debug.sh"
[ -r "$_DATA/local.opts" ] && . "$_DATA/local.opts"

if [ "$1" = '--server' -o  "$1" = '--inetd' -o "$1" = '--ncat' ]; then
  . "$_EXEC/shcgi/server.sh" $@
else
  HTTP_format(){ cat }
fi

. "$_EXEC/shcgi/cgi.sh"

[ -x "$_EXEC/globals.sh" ] && . "$_EXEC/globals.sh"

  PAGE=$(validate   "${PAGE:-${_GET[page]}}"   '[a-zA-Z0-9_-]+' '')
ACTION=$(validate "${ACTION:-${_GET[action]}}" '[a-zA-Z0-9_-]+' '')
STATIC=$(invalidate "${STATIC:-${_GET[static]}}" '(^|.*/)\.\./.*' '')

if [ -n "$STATIC" ] && [ -e "${_EXEC}/static/${STATIC}" ]; then
  . "$_EXEC/shcgi/static.sh" "$_EXEC/static/$STATIC"
elif [ -n "$ACTION" ] && [ -x "${_EXEC}/actions/${ACTION}.sh" ]; then
  . "${_EXEC}/actions/${ACTION}.sh"
elif   [ -n "$PAGE" ] && [ -x "${_EXEC}/pages/${PAGE}.sh" ]; then
  . "$_EXEC/shcgi/page.sh"
else
  printf 'HTTP/1.1 404 Not Found\r\n'
  PAGE=error
  [ -x "${_EXEC}/pages/${PAGE}.sh" ] && . "$_EXEC/shcgi/page.sh"
fi |HTTP_format $@
